@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@ class CControllerMenuPopup extends CController {
 
 	protected function checkInput() {
 		$fields = [
-			'type' => 'required|in dashboard,history,host,item,item_prototype,map_element,refresh,trigger,trigger_macro',
+			'type' => 'required|in dashboard,history,host,item,item_prototype,map_element,refresh,trigger,trigger_macro,widget_actions',
 			'data' => 'array'
 		];
 
@@ -35,7 +35,7 @@ class CControllerMenuPopup extends CController {
 				$output['errors'] = $messages->toString();
 			}
 
-			$this->setResponse(new CControllerResponseData(['main_block' => CJs::encodeJson($output)]));
+			$this->setResponse(new CControllerResponseData(['main_block' => json_encode($output)]));
 		}
 
 		return $ret;
@@ -128,18 +128,28 @@ class CControllerMenuPopup extends CController {
 
 		$db_hosts = $has_goto
 			? API::Host()->get([
-				'output' => ['status'],
+				'output' => ['hostid', 'status'],
 				'selectGraphs' => API_OUTPUT_COUNT,
 				'selectScreens' => API_OUTPUT_COUNT,
+				'selectHttpTests' => API_OUTPUT_COUNT,
 				'hostids' => $data['hostid']
 			])
 			: API::Host()->get([
-				'output' => [],
+				'output' => ['hostid'],
 				'hostids' => $data['hostid']
 			]);
 
 		if ($db_hosts) {
 			$db_host = $db_hosts[0];
+			$rw_hosts = false;
+
+			if ($has_goto && CWebUser::getType() > USER_TYPE_ZABBIX_USER) {
+				$rw_hosts = (bool) API::Host()->get([
+					'output' => [],
+					'hostids' => $db_host['hostid'],
+					'editable' => true
+				]);
+			}
 
 			$scripts = API::Script()->getScriptsByHosts([$data['hostid']])[$data['hostid']];
 
@@ -159,9 +169,12 @@ class CControllerMenuPopup extends CController {
 			if ($has_goto) {
 				$menu_data['showGraphs'] = (bool) $db_host['graphs'];
 				$menu_data['showScreens'] = (bool) $db_host['screens'];
+				$menu_data['showWeb'] = (bool) $db_host['httpTests'];
+				$menu_data['showConfig'] = (CWebUser::getType() > USER_TYPE_ZABBIX_USER);
+				$menu_data['isWriteable'] = $rw_hosts;
 				$menu_data['showTriggers'] = ($db_host['status'] == HOST_STATUS_MONITORED);
 				if (array_key_exists('severity_min', $data)) {
-					$menu_data['severity_min'] = $data['severity_min'];
+					$menu_data['severities'] = array_column(getSeverities($data['severity_min']), 'value');
 				}
 				if (array_key_exists('show_suppressed', $data)) {
 					$menu_data['show_suppressed'] = $data['show_suppressed'];
@@ -394,7 +407,7 @@ class CControllerMenuPopup extends CController {
 							'groupid' => $selement['elements'][0]['groupid']
 						];
 						if (array_key_exists('severity_min', $data)) {
-							$menu_data['severity_min'] = $data['severity_min'];
+							$menu_data['severities'] = array_column(getSeverities($data['severity_min']), 'value');
 						}
 						if ($db_map['show_suppressed']) {
 							$menu_data['show_suppressed'] = true;
@@ -431,7 +444,7 @@ class CControllerMenuPopup extends CController {
 							'triggerids' => zbx_objectValues($selement['elements'], 'triggerid')
 						];
 						if (array_key_exists('severity_min', $data)) {
-							$menu_data['severity_min'] = $data['severity_min'];
+							$menu_data['severities'] = array_column(getSeverities($data['severity_min']), 'value');
 						}
 						if ($db_map['show_suppressed']) {
 							$menu_data['show_suppressed'] = true;
@@ -490,8 +503,7 @@ class CControllerMenuPopup extends CController {
 	 * @param array  $data
 	 * @param string $data['triggerid']
 	 * @param string $data['eventid']                 (optional) Mandatory for Acknowledge menu.
-	 * @param array  $data['acknowledge']             (optional) Acknowledge link parameters.
-	 * @param string $data['acknowledge']['backurl']
+	 * @param bool   $data['acknowledge']             (optional) Whether to show Acknowledge menu.
 	 * @param int    $data['severity_min']            (optional)
 	 * @param bool   $data['show_suppressed']         (optional)
 	 * @param array  $data['urls']                    (optional)
@@ -622,6 +634,49 @@ class CControllerMenuPopup extends CController {
 		return ['type' => 'trigger_macro'];
 	}
 
+	/**
+	 * Prepare data for widget actions menu popup.
+	 *
+	 * @param array  $data
+	 * @param string $data['widget_uniqueid']  Widget instance unique id.
+	 * @param string $data['currentRate']      Refresh rate for widget.
+	 * @param bool   $data['multiplier']       Multiplier or time mode.
+	 *
+	 * @return mixed
+	 */
+	private static function getMenuDataWidgetActions(array $data) {
+		$menu_data = [
+			'type' => 'widget_actions',
+			'widget_uniqueid' => $data['widget_uniqueid'],
+			'currentRate' => $data['currentRate'],
+			'multiplier' => (bool) $data['multiplier']
+		];
+
+		if ($data['widgetType'] == WIDGET_SVG_GRAPH) {
+			$menu_data['download'] = true;
+		}
+
+		if (array_key_exists('params', $data)) {
+			$menu_data['params'] = $data['params'];
+		}
+
+		if ($data['widgetType'] == WIDGET_GRAPH && array_key_exists('graphid', $data) && $data['graphid']) {
+			$menu_data['download'] = (bool) API::Graph()->get([
+				'output' => ['graphid'],
+				'graphids' => $data['graphid']
+			]);
+		}
+		elseif ($data['widgetType'] == WIDGET_GRAPH && array_key_exists('itemid', $data) && $data['itemid']) {
+			$menu_data['download'] = (bool) API::Item()->get([
+				'output' => ['itemid'],
+				'itemids' => $data['itemid'],
+				'webitems' => true
+			]);
+		}
+
+		return $menu_data;
+	}
+
 	protected function doAction() {
 		$data = $this->hasInput('data') ? $this->getInput('data') : [];
 
@@ -661,6 +716,10 @@ class CControllerMenuPopup extends CController {
 			case 'trigger_macro':
 				$menu_data = self::getMenuDataTriggerMacro();
 				break;
+
+			case 'widget_actions':
+				$menu_data = self::getMenuDataWidgetActions($data);
+				break;
 		}
 
 		$output = [];
@@ -673,6 +732,6 @@ class CControllerMenuPopup extends CController {
 			$output['errors'] = $messages->toString();
 		}
 
-		$this->setResponse(new CControllerResponseData(['main_block' => CJs::encodeJson($output)]));
+		$this->setResponse(new CControllerResponseData(['main_block' => json_encode($output)]));
 	}
 }
