@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,38 +24,74 @@
  */
 class CHostPrototype extends CHostBase {
 
-	protected $sortColumns = ['hostid', 'host', 'name', 'status'];
-
-	public function __construct() {
-		parent::__construct();
-
-		$this->getOptions = array_merge($this->getOptions, [
-			'discoveryids'  		=> null,
-			'inherited'				=> null,
-			'selectDiscoveryRule' 	=> null,
-			'selectGroupLinks'		=> null,
-			'selectGroupPrototypes' => null,
-			'selectParentHost'		=> null,
-			'selectTemplates' 		=> null,
-			'selectInventory' 		=> null,
-			'editable'				=> false,
-			'nopermissions'			=> null,
-			'sortfield'    			=> '',
-			'sortorder'     		=> ''
-		]);
-	}
+	protected $sortColumns = ['hostid', 'host', 'name', 'status', 'discover'];
 
 	/**
 	 * Get host prototypes.
 	 *
 	 * @param array $options
+	 * @param bool  $options['selectMacros']  Array of macros fields to be selected or string "extend".
 	 *
 	 * @return array
 	 */
 	public function get(array $options) {
-		$options = zbx_array_merge($this->getOptions, $options);
+		$hosts_fields = array_keys($this->getTableSchema('hosts')['fields']);
+		$output_fields = ['hostid', 'host', 'name', 'status', 'templateid', 'inventory_mode', 'discover'];
+		$link_fields = ['group_prototypeid', 'groupid', 'hostid', 'templateid'];
+		$group_fields = ['group_prototypeid', 'name', 'hostid', 'templateid'];
+		$discovery_fields = array_keys($this->getTableSchema('items')['fields']);
+		$hostmacro_fields = array_keys($this->getTableSchema('hostmacro')['fields']);
+
+		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
+			// filter
+			'hostids' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'default' => null],
+			'discoveryids' =>			['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'default' => null],
+			'filter' =>					['type' => API_OBJECT, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => [
+				'hostid' =>					['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'host' =>					['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'name' =>					['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'status' =>					['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'in' => implode(',', [HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED])],
+				'templateid' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'inventory_mode' =>			['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'in' => implode(',', [HOST_INVENTORY_DISABLED, HOST_INVENTORY_MANUAL, HOST_INVENTORY_AUTOMATIC])]
+			]],
+			'search' =>					['type' => API_OBJECT, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => [
+				'host' =>					['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
+				'name' =>					['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE]
+			]],
+			'searchByAny' =>			['type' => API_BOOLEAN, 'default' => false],
+			'startSearch' =>			['type' => API_FLAG, 'default' => false],
+			'excludeSearch' =>			['type' => API_FLAG, 'default' => false],
+			'searchWildcardsEnabled' =>	['type' => API_BOOLEAN, 'default' => false],
+			// output
+			'output' =>					['type' => API_OUTPUT, 'in' => 'inventory_mode,'.implode(',', $output_fields), 'default' => $output_fields],
+			'countOutput' =>			['type' => API_FLAG, 'default' => false],
+			'groupCount' =>				['type' => API_FLAG, 'default' => false],
+			'selectGroupLinks' =>		['type' => API_OUTPUT, 'flags' => API_ALLOW_NULL, 'in' => implode(',', $link_fields), 'default' => null],
+			'selectGroupPrototypes' =>	['type' => API_OUTPUT, 'flags' => API_ALLOW_NULL, 'in' => implode(',', $group_fields), 'default' => null],
+			'selectDiscoveryRule' =>	['type' => API_OUTPUT, 'flags' => API_ALLOW_NULL, 'in' => implode(',', $discovery_fields), 'default' => null],
+			'selectParentHost' =>		['type' => API_OUTPUT, 'flags' => API_ALLOW_NULL, 'in' => implode(',', $hosts_fields), 'default' => null],
+			'selectTemplates' =>		['type' => API_OUTPUT, 'flags' => API_ALLOW_NULL | API_ALLOW_COUNT, 'in' => implode(',', $hosts_fields), 'default' => null],
+			'selectMacros' =>			['type' => API_OUTPUT, 'flags' => API_ALLOW_NULL, 'in' => implode(',', $hostmacro_fields), 'default' => null],
+			// sort and limit
+			'sortfield' =>				['type' => API_STRINGS_UTF8, 'flags' => API_NORMALIZE, 'in' => implode(',', $this->sortColumns), 'uniq' => true, 'default' => []],
+			'sortorder' =>				['type' => API_SORTORDER, 'default' => []],
+			'limit' =>					['type' => API_INT32, 'flags' => API_ALLOW_NULL, 'in' => '1:'.ZBX_MAX_INT32, 'default' => null],
+			// flags
+			'inherited'	=>				['type' => API_BOOLEAN, 'flags' => API_ALLOW_NULL, 'default' => null],
+			'editable' =>				['type' => API_BOOLEAN, 'default' => false],
+			'preservekeys' =>			['type' => API_BOOLEAN, 'default' => false],
+			'nopermissions' =>			['type' => API_BOOLEAN, 'default' => false]	// TODO: This property and frontend usage SHOULD BE removed.
+		]];
+		if (!CApiInputValidator::validate($api_input_rules, $options, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+
 		$options['filter']['flags'] = ZBX_FLAG_DISCOVERY_PROTOTYPE;
 
+		if ($options['output'] === API_OUTPUT_EXTEND) {
+			$options['output'] = $output_fields;
+		}
+
 		// build and execute query
 		$sql = $this->createSelectQuery($this->tableName(), $options);
 		$res = DBselect($sql, $options['limit']);
@@ -94,6 +130,28 @@ class CHostPrototype extends CHostBase {
 		return $result;
 	}
 
+	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
+		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
+
+		if (!$options['countOutput'] && $this->outputIsRequested('inventory_mode', $options['output'])) {
+			$sqlParts['select']['inventory_mode'] =
+				dbConditionCoalesce('hinv.inventory_mode', HOST_INVENTORY_DISABLED, 'inventory_mode');
+		}
+
+		if ((!$options['countOutput'] && $this->outputIsRequested('inventory_mode', $options['output']))
+				|| ($options['filter'] && array_key_exists('inventory_mode', $options['filter']))) {
+			$sqlParts['left_join'][] = [
+				'from' => 'host_inventory hinv',
+				'on' => $this->tableAlias().'.'.$this->pk().'=hinv.hostid'
+			];
+
+			$sqlParts['left_table'] = array_search($this->tableName().' '.$this->tableAlias(), $sqlParts['from']);
+		}
+
+		return $sqlParts;
+	}
+
+
 	/**
 	 * Check for duplicated names.
 	 *
@@ -141,17 +199,22 @@ class CHostPrototype extends CHostBase {
 			'host' =>				['type' => API_H_NAME, 'flags' => API_REQUIRED | API_REQUIRED_LLD_MACRO, 'length' => DB::getFieldLength('hosts', 'host')],
 			'name' =>				['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('hosts', 'name'), 'default_source' => 'host'],
 			'status' =>				['type' => API_INT32, 'in' => implode(',', [HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED])],
+			'discover' =>			['type' => API_INT32, 'in' => implode(',', [HOST_DISCOVER, HOST_NO_DISCOVER])],
 			'groupLinks' =>			['type' => API_OBJECTS, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'uniq' => [['groupid']], 'fields' => [
 				'groupid' =>			['type' => API_ID, 'flags' => API_REQUIRED]
 			]],
 			'groupPrototypes' =>	['type' => API_OBJECTS, 'uniq' => [['name']], 'fields' => [
 				'name' =>				['type' => API_HG_NAME, 'flags' => API_REQUIRED | API_REQUIRED_LLD_MACRO, 'length' => DB::getFieldLength('hstgrp', 'name')]
 			]],
-			'inventory' =>			['type' => API_OBJECT, 'fields' => [
-				'inventory_mode' =>		['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [HOST_INVENTORY_DISABLED, HOST_INVENTORY_MANUAL, HOST_INVENTORY_AUTOMATIC])]
-			]],
+			'inventory_mode' =>		['type' => API_INT32, 'in' => implode(',', [HOST_INVENTORY_DISABLED, HOST_INVENTORY_MANUAL, HOST_INVENTORY_AUTOMATIC])],
 			'templates' =>			['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['templateid']], 'fields' => [
 				'templateid' =>			['type' => API_ID, 'flags' => API_REQUIRED]
+			]],
+			'macros' =>				['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['macro']], 'fields' => [
+				'macro' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY],
+				'value' =>				['type' => API_STRING_UTF8, 'flag' => API_REQUIRED | API_NOT_EMPTY],
+				'type' =>				['type' => API_INT32, 'flag' => API_REQUIRED, 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET])],
+				'description' => 		['type' => API_STRING_UTF8]
 			]]
 		]];
 		if (!CApiInputValidator::validate($api_input_rules, $host_prototypes, '/', $error)) {
@@ -220,6 +283,7 @@ class CHostPrototype extends CHostBase {
 		unset($host_prototype);
 
 		$host_prototypes = $this->createReal($host_prototypes);
+		$this->createMacros(array_column($host_prototypes, 'macros', 'hostid'));
 		$this->inherit($host_prototypes);
 
 		$this->addAuditBulk(AUDIT_ACTION_ADD, AUDIT_RESOURCE_HOST_PROTOTYPE, $host_prototypes);
@@ -262,12 +326,11 @@ class CHostPrototype extends CHostBase {
 			];
 
 			// inventory
-			if (isset($hostPrototype['inventory']['inventory_mode'])
-					&& ($hostPrototype['inventory']['inventory_mode'] == HOST_INVENTORY_MANUAL
-						|| $hostPrototype['inventory']['inventory_mode'] == HOST_INVENTORY_AUTOMATIC)) {
+			if (array_key_exists('inventory_mode', $hostPrototype)
+					&& $hostPrototype['inventory_mode'] != HOST_INVENTORY_DISABLED) {
 				$hostPrototypeInventory[] = [
 					'hostid' => $hostPrototype['hostid'],
-					'inventory_mode' => $hostPrototype['inventory']['inventory_mode']
+					'inventory_mode' => $hostPrototype['inventory_mode']
 				];
 			}
 		}
@@ -288,7 +351,7 @@ class CHostPrototype extends CHostBase {
 		DB::insert('host_discovery', $hostPrototypeDiscoveryRules, false);
 
 		// save inventory
-		DB::insert('host_inventory', $hostPrototypeInventory, false);
+		DB::insertBatch('host_inventory', $hostPrototypeInventory, false);
 
 		// link templates
 		foreach ($hostPrototypes as $hostPrototype) {
@@ -314,6 +377,7 @@ class CHostPrototype extends CHostBase {
 			'host' =>				['type' => API_H_NAME, 'flags' => API_REQUIRED_LLD_MACRO, 'length' => DB::getFieldLength('hosts', 'host')],
 			'name' =>				['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('hosts', 'name')],
 			'status' =>				['type' => API_INT32, 'in' => implode(',', [HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED])],
+			'discover' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_PROTOTYPE_DISCOVER, ZBX_PROTOTYPE_NO_DISCOVER])],
 			'groupLinks' =>			['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY, 'uniq' => [['group_prototypeid'], ['groupid']], 'fields' => [
 				'group_prototypeid' =>	['type' => API_ID],
 				'groupid' =>			['type' => API_ID]
@@ -322,11 +386,16 @@ class CHostPrototype extends CHostBase {
 				'group_prototypeid' =>	['type' => API_ID],
 				'name' =>				['type' => API_HG_NAME, 'flags' => API_REQUIRED_LLD_MACRO, 'length' => DB::getFieldLength('hstgrp', 'name')]
 			]],
-			'inventory' =>			['type' => API_OBJECT, 'fields' => [
-				'inventory_mode' =>		['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [HOST_INVENTORY_DISABLED, HOST_INVENTORY_MANUAL, HOST_INVENTORY_AUTOMATIC])]
-			]],
+			'inventory_mode' =>		['type' => API_INT32, 'in' => implode(',', [HOST_INVENTORY_DISABLED, HOST_INVENTORY_MANUAL, HOST_INVENTORY_AUTOMATIC])],
 			'templates' =>			['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['templateid']], 'fields' => [
 				'templateid' =>			['type' => API_ID, 'flags' => API_REQUIRED]
+			]],
+			'macros' =>				['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['macro']], 'fields' => [
+				'hostmacroid' =>		['type' => API_ID],
+				'macro' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY],
+				'value' =>				['type' => API_STRING_UTF8],
+				'type' =>				['type' => API_INT32, 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET])],
+				'description' => 		['type' => API_STRING_UTF8]
 			]]
 		]];
 		if (!CApiInputValidator::validate($api_input_rules, $host_prototypes, '/', $error)) {
@@ -461,6 +530,12 @@ class CHostPrototype extends CHostBase {
 		}
 		unset($host_prototype);
 
+		$macros = array_column($host_prototypes, 'macros', 'hostid');
+
+		if ($macros) {
+			$this->updateMacros($macros);
+		}
+
 		$host_prototypes = $this->updateReal($host_prototypes);
 		$this->inherit($host_prototypes);
 
@@ -479,95 +554,93 @@ class CHostPrototype extends CHostBase {
 	/**
 	 * Updates the host prototypes and propagates the changes to linked hosts and templates.
 	 *
-	 * @param array $hostPrototypes
+	 * @param array $host_prototypes
 	 *
 	 * @return array
 	 */
-	protected function updateReal(array $hostPrototypes) {
+	protected function updateReal(array $host_prototypes) {
 		// save the host prototypes
-		foreach ($hostPrototypes as $hostPrototype) {
-			DB::updateByPk($this->tableName(), $hostPrototype['hostid'], $hostPrototype);
+		foreach ($host_prototypes as $host_prototype) {
+			DB::updateByPk($this->tableName(), $host_prototype['hostid'], $host_prototype);
 		}
 
-		$exHostPrototypes = $this->get([
-			'output' => ['hostid'],
+		$ex_host_prototypes = $this->get([
+			'output' => ['hostid', 'inventory_mode'],
 			'selectGroupLinks' => API_OUTPUT_EXTEND,
 			'selectGroupPrototypes' => API_OUTPUT_EXTEND,
 			'selectTemplates' => ['templateid'],
-			'selectInventory' => API_OUTPUT_EXTEND,
-			'hostids' => zbx_objectValues($hostPrototypes, 'hostid'),
+			'hostids' => zbx_objectValues($host_prototypes, 'hostid'),
 			'preservekeys' => true
 		]);
 
 		// update related objects
-		$inventoryCreate = [];
-		$inventoryDeleteIds = [];
-		foreach ($hostPrototypes as $key => $hostPrototype) {
-			$exHostPrototype = $exHostPrototypes[$hostPrototype['hostid']];
+		$inventory_create = [];
+		$inventory_deleteids = [];
+		foreach ($host_prototypes as $key => $host_prototype) {
+			$ex_host_prototype = $ex_host_prototypes[$host_prototype['hostid']];
 
 			// group prototypes
-			if (isset($hostPrototype['groupPrototypes'])) {
-				foreach ($hostPrototype['groupPrototypes'] as &$groupPrototype) {
-					$groupPrototype['hostid'] = $hostPrototype['hostid'];
+			if (isset($host_prototype['groupPrototypes'])) {
+				foreach ($host_prototype['groupPrototypes'] as &$group_prototype) {
+					$group_prototype['hostid'] = $host_prototype['hostid'];
 				}
-				unset($groupPrototype);
+				unset($group_prototype);
 
 				// save group prototypes
-				$exGroupPrototypes = zbx_toHash(
-					array_merge($exHostPrototype['groupLinks'], $exHostPrototype['groupPrototypes']),
+				$ex_group_prototypes = zbx_toHash(
+					array_merge($ex_host_prototype['groupLinks'], $ex_host_prototype['groupPrototypes']),
 					'group_prototypeid'
 				);
-				$modifiedGroupPrototypes = [];
-				foreach ($hostPrototype['groupPrototypes'] as $groupPrototype) {
-					if (isset($groupPrototype['group_prototypeid'])) {
-						unset($exGroupPrototypes[$groupPrototype['group_prototypeid']]);
+				$modified_group_prototypes = [];
+				foreach ($host_prototype['groupPrototypes'] as $group_prototype) {
+					if (isset($group_prototype['group_prototypeid'])) {
+						unset($ex_group_prototypes[$group_prototype['group_prototypeid']]);
 					}
 
-					$modifiedGroupPrototypes[] = $groupPrototype;
+					$modified_group_prototypes[] = $group_prototype;
 				}
-				if ($exGroupPrototypes) {
-					$this->deleteGroupPrototypes(array_keys($exGroupPrototypes));
+				if ($ex_group_prototypes) {
+					$this->deleteGroupPrototypes(array_keys($ex_group_prototypes));
 				}
-				$hostPrototypes[$key]['groupPrototypes'] = DB::save('group_prototype', $modifiedGroupPrototypes);
+				$host_prototypes[$key]['groupPrototypes'] = DB::save('group_prototype', $modified_group_prototypes);
 			}
 
 			// templates
-			if (isset($hostPrototype['templates'])) {
-				$existingTemplateIds = zbx_objectValues($exHostPrototype['templates'], 'templateid');
-				$newTemplateIds = zbx_objectValues($hostPrototype['templates'], 'templateid');
-				$this->unlink(array_diff($existingTemplateIds, $newTemplateIds), [$hostPrototype['hostid']]);
-				$this->link(array_diff($newTemplateIds, $existingTemplateIds), [$hostPrototype['hostid']]);
+			if (isset($host_prototype['templates'])) {
+				$existing_templateids = zbx_objectValues($ex_host_prototype['templates'], 'templateid');
+				$new_templateids = zbx_objectValues($host_prototype['templates'], 'templateid');
+				$this->unlink(array_diff($existing_templateids, $new_templateids), [$host_prototype['hostid']]);
+				$this->link(array_diff($new_templateids, $existing_templateids), [$host_prototype['hostid']]);
 			}
 
 			// inventory
-			if (isset($hostPrototype['inventory']) ) {
-				$inventory = zbx_array_mintersect(['inventory_mode'], $hostPrototype['inventory']);
-
-				if (array_key_exists('inventory_mode', $inventory)
-					&& ($inventory['inventory_mode'] == HOST_INVENTORY_MANUAL
-						|| $inventory['inventory_mode'] == HOST_INVENTORY_AUTOMATIC)) {
-
-					if ($exHostPrototype['inventory']['inventory_mode'] != HOST_INVENTORY_DISABLED) {
-						DB::update('host_inventory', [
-							'values' => $inventory,
-							'where' => ['hostid' => $hostPrototype['hostid']]
-						]);
-					}
-					else {
-						$inventoryCreate[] = $inventory + ['hostid' => $hostPrototype['hostid']];
-					}
+			if (array_key_exists('inventory_mode', $host_prototype)) {
+				if ($host_prototype['inventory_mode'] == HOST_INVENTORY_DISABLED) {
+					$inventory_deleteids[] = $host_prototype['hostid'];
 				}
 				else {
-					$inventoryDeleteIds[] = $hostPrototype['hostid'];
+					$inventory = ['inventory_mode' => $host_prototype['inventory_mode']];
+
+					if ($ex_host_prototype['inventory_mode'] != HOST_INVENTORY_DISABLED) {
+						if ($host_prototype['inventory_mode'] != $ex_host_prototype['inventory_mode']) {
+							DB::update('host_inventory', [
+								'values' => $inventory,
+								'where' => ['hostid' => $host_prototype['hostid']]
+							]);
+						}
+					}
+					else {
+						$inventory_create[] = $inventory + ['hostid' => $host_prototype['hostid']];
+					}
 				}
 			}
 		}
 
 		// save inventory
-		DB::insert('host_inventory', $inventoryCreate, false);
-		DB::delete('host_inventory', ['hostid' => $inventoryDeleteIds]);
+		DB::insertBatch('host_inventory', $inventory_create, false);
+		DB::delete('host_inventory', ['hostid' => $inventory_deleteids]);
 
-		return $hostPrototypes;
+		return $host_prototypes;
 	}
 
 	/**
@@ -604,10 +677,16 @@ class CHostPrototype extends CHostBase {
 		// save the new host prototypes
 		if (!zbx_empty($insertHostPrototypes)) {
 			$insertHostPrototypes = $this->createReal($insertHostPrototypes);
+			$this->createMacros(array_column($insertHostPrototypes, 'macros', 'hostid'));
 		}
 
 		if (!zbx_empty($updateHostPrototypes)) {
 			$updateHostPrototypes = $this->updateReal($updateHostPrototypes);
+			$macros = array_column($updateHostPrototypes, 'macros', 'hostid');
+
+			if ($macros) {
+				$this->updateMacros($macros);
+			}
 		}
 
 		$host_prototypes = array_merge($updateHostPrototypes, $insertHostPrototypes);
@@ -839,9 +918,9 @@ class CHostPrototype extends CHostBase {
 			'output' => API_OUTPUT_EXTEND,
 			'selectGroupLinks' => API_OUTPUT_EXTEND,
 			'selectGroupPrototypes' => API_OUTPUT_EXTEND,
+			'selectMacros' => ['macro', 'type', 'value', 'description'],
 			'selectTemplates' => ['templateid'],
 			'selectDiscoveryRule' => ['itemid'],
-			'selectInventory' => ['inventory_mode']
 		]);
 
 		foreach ($hostPrototypes as &$hostPrototype) {
@@ -906,6 +985,14 @@ class CHostPrototype extends CHostBase {
 
 		$hostPrototypeIds = array_merge($hostPrototypeIds, $childHostPrototypeIds);
 
+		// Lock host prototypes before delete to prevent server from adding new LLD hosts.
+		DBselect(
+			'SELECT NULL'.
+			' FROM hosts h'.
+			' WHERE '.dbConditionInt('h.hostid', $hostPrototypeIds).
+			' FOR UPDATE'
+		);
+
 		$deleteHostPrototypes = $this->get([
 			'hostids' => $hostPrototypeIds,
 			'output' => ['host'],
@@ -1117,6 +1204,28 @@ class CHostPrototype extends CHostBase {
 			$sqlParts['where'][] = ($options['inherited']) ? 'h.templateid IS NOT NULL' : 'h.templateid IS NULL';
 		}
 
+		if ($options['filter'] && array_key_exists('inventory_mode', $options['filter'])) {
+			if ($options['filter']['inventory_mode'] !== null) {
+				$inventory_mode_query = (array) $options['filter']['inventory_mode'];
+
+				$inventory_mode_where = [];
+				$null_position = array_search(HOST_INVENTORY_DISABLED, $inventory_mode_query);
+
+				if ($null_position !== false) {
+					unset($inventory_mode_query[$null_position]);
+					$inventory_mode_where[] = 'hinv.inventory_mode IS NULL';
+				}
+
+				if ($null_position === false || $inventory_mode_query) {
+					$inventory_mode_where[] = dbConditionInt('hinv.inventory_mode', $inventory_mode_query);
+				}
+
+				$sqlParts['where'][] = (count($inventory_mode_where) > 1)
+					? '('.implode(' OR ', $inventory_mode_where).')'
+					: $inventory_mode_where[0];
+			}
+		}
+
 		return $sqlParts;
 	}
 
@@ -1236,27 +1345,17 @@ class CHostPrototype extends CHostBase {
 			}
 		}
 
-		// adding inventory
-		if ($options['selectInventory'] !== null) {
-			$inventory = API::getApiService()->select('host_inventory', [
-				'output' => ['hostid', 'inventory_mode'],
-				'filter' => ['hostid' => $hostPrototypeIds],
+		// adding macros
+		if ($options['selectMacros'] !== null && $options['selectMacros'] != API_OUTPUT_COUNT) {
+			$macros = API::UserMacro()->get([
+				'output' => $this->outputExtend($options['selectMacros'], ['hostid', 'hostmacroid']),
+				'hostids' => $hostPrototypeIds,
 				'preservekeys' => true
 			]);
 
-			foreach ($hostPrototypeIds as $host_prototypeid) {
-				// There is no DB record if inventory mode is HOST_INVENTORY_DISABLED.
-				if (!array_key_exists($host_prototypeid, $inventory)) {
-					$inventory[$host_prototypeid] = [
-						'hostid' => (string) $host_prototypeid,
-						'inventory_mode' => (string) HOST_INVENTORY_DISABLED
-					];
-				}
-			}
-
-			$relation_map = $this->createRelationMap($result, 'hostid', 'hostid');
-			$inventory = $this->unsetExtraFields($inventory, ['hostid', 'inventory_mode'], $options['selectInventory']);
-			$result = $relation_map->mapOne($result, $inventory, 'inventory');
+			$relationMap = $this->createRelationMap($macros, 'hostid', 'hostmacroid');
+			$macros = $this->unsetExtraFields($macros, ['hostid', 'hostmacroid'], $options['selectMacros']);
+			$result = $relationMap->mapMany($result, $macros, 'macros');
 		}
 
 		return $result;
@@ -1269,6 +1368,14 @@ class CHostPrototype extends CHostBase {
 	 * @param array $groupPrototypeIds
 	 */
 	protected function deleteGroupPrototypes(array $groupPrototypeIds) {
+		// Lock group prototypes before delete to prevent server from adding new LLD elements.
+		DBselect(
+			'SELECT NULL'.
+			' FROM group_prototype gp'.
+			' WHERE '.dbConditionInt('gp.group_prototypeid', $groupPrototypeIds).
+			' FOR UPDATE'
+		);
+
 		// delete child group prototypes
 		$groupPrototypeChildren = DBfetchArray(DBselect(
 			'SELECT gp.group_prototypeid FROM group_prototype gp WHERE '.dbConditionInt('templateid', $groupPrototypeIds)
@@ -1288,4 +1395,75 @@ class CHostPrototype extends CHostBase {
 		// delete group prototypes
 		DB::delete('group_prototype', ['group_prototypeid' => $groupPrototypeIds]);
 	}
+
+	/**
+	 * Create host prototype macro with prototype id as key and macros array.
+	 *
+	 * @param array $host_prototype_macro  Array of host prototype macros.
+	 */
+	protected function createMacros(array $host_prototype_macros) {
+		$create = [];
+
+		foreach ($host_prototype_macros as $host_prototypeid => $macros) {
+			foreach ($macros as $macro) {
+				$create[] = ['hostid' => $host_prototypeid] + $macro;
+			}
+		}
+
+		if ($create) {
+			API::UserMacro()->create($create);
+		}
+	}
+
+	/**
+	 * Update host prototype macros, key is host prototype id and value is array of arrays with macro objects.
+	 *
+	 * @param array $macros     Array with macros objects.
+	 */
+	protected function updateMacros(array $update_macros) {
+		$create = [];
+		$update = [];
+		$db_macros = API::UserMacro()->get([
+			'output' => ['hostid', 'macro', 'type', 'value', 'description'],
+			'hostids' => array_keys($update_macros),
+			'preservekeys' => true
+		]);
+		$host_macros = array_fill_keys(array_column($db_macros, 'hostid'), []);
+
+		foreach ($db_macros as $hostmacroid => $db_macro) {
+			$host_macros[$db_macro['hostid']][$db_macro['macro']] = $hostmacroid;
+		}
+
+		foreach ($update_macros as $hostid => $macros) {
+			foreach ($macros as $macro) {
+				if (array_key_exists($hostid, $host_macros)
+						&& array_key_exists($macro['macro'], $host_macros[$hostid])) {
+					$hostmacroid = $host_macros[$hostid][$macro['macro']];
+					$diff = array_diff($macro, $db_macros[$hostmacroid]);
+					unset($diff['hostid'], $diff['hostmacroid']);
+
+					if ($diff) {
+						$update[] = ['hostmacroid' => $hostmacroid] + $diff;
+					}
+
+					unset($db_macros[$hostmacroid], $host_macros[$hostid][$macro['macro']]);
+				}
+				else {
+					$create[] = ['hostid' => $hostid] + $macro;
+				}
+			}
+		}
+
+		if ($create) {
+			API::UserMacro()->create($create);
+		}
+
+		if ($update) {
+			API::UserMacro()->update($update);
+		}
+
+		if ($db_macros) {
+			API::UserMacro()->delete(array_keys($db_macros));
+		}
+	}
 }
