@@ -75,7 +75,8 @@ class CSvgGraph extends CSvg {
 	protected $points = [];
 
 	/**
-	 * Array of metric paths. Where key is metric index from $metrics array.
+	 * Array of metric paths. Where key is metric index from $metrics array, and value is either array of paths or
+	 * single path (in case of bar and points graphs).
 	 *
 	 * @var array
 	 */
@@ -95,14 +96,24 @@ class CSvgGraph extends CSvg {
 
 	protected $left_y_show = false;
 	protected $left_y_min = null;
+	protected $left_y_min_calculated = null;
 	protected $left_y_max = null;
+	protected $left_y_max_calculated = null;
+	protected $left_y_interval = null;
 	protected $left_y_units = null;
+	protected $left_y_is_binary = null;
+	protected $left_y_power = null;
 	protected $left_y_empty = true;
 
 	protected $right_y_show = false;
 	protected $right_y_min = null;
+	protected $right_y_min_calculated = null;
 	protected $right_y_max = null;
+	protected $right_y_max_calculated = null;
+	protected $right_y_interval = null;
 	protected $right_y_units = null;
+	protected $right_y_is_binary = null;
+	protected $right_y_power = null;
 	protected $right_y_empty = true;
 
 	protected $right_y_zero = null;
@@ -133,6 +144,8 @@ class CSvgGraph extends CSvg {
 	 */
 	protected $max_yaxis_width = 120;
 
+	protected $cell_height_min = 30;
+
 	protected $offset_top;
 	protected $time_from;
 	protected $time_till;
@@ -236,11 +249,14 @@ class CSvgGraph extends CSvg {
 				$metrics_for_each_axes[$metric['options']['axisy']]++;
 
 				foreach ($metric['points'] as $point) {
-					if ($min_value === null || $min_value > $point['value']) {
-						$min_value = $point['value'];
-					}
-					if ($max_value === null || $max_value < $point['value']) {
-						$max_value = $point['value'];
+					// Point with value of null is used as marker in denoting missing point. It should not affect axis.
+					if ($point['value'] !== null) {
+						if ($min_value === null || $min_value > $point['value']) {
+							$min_value = $point['value'];
+						}
+						if ($max_value === null || $max_value < $point['value']) {
+							$max_value = $point['value'];
+						}
 					}
 
 					$this->points[$i][$point['clock']] = $point['value'];
@@ -370,7 +386,7 @@ class CSvgGraph extends CSvg {
 	 */
 	public function getTimeGridWithPosition() {
 		$period = $this->time_till - $this->time_from;
-		$step = round(bcmul(bcdiv($period, $this->canvas_width), 100)); // Grid cell (100px) in seconds.
+		$step = round($period / $this->canvas_width * 100); // Grid cell (100px) in seconds.
 
 		/*
 		 * In case if requested time period is so small that it is rounded to zero, we are displaying only two
@@ -438,7 +454,6 @@ class CSvgGraph extends CSvg {
 	 * @return CSvgGraph
 	 */
 	public function draw() {
-		$this->applyMissingDataFunc();
 		$this->calculateDimensions();
 		$this->calculatePaths();
 
@@ -494,30 +509,13 @@ class CSvgGraph extends CSvg {
 	 */
 	protected function calculateDimensions() {
 		// Canvas height must be specified before call self::getValuesGridWithPosition.
+
 		$this->offset_top = 10;
 		$this->offset_bottom = self::SVG_GRAPH_X_AXIS_HEIGHT;
 		$this->canvas_height = $this->height - $this->offset_top - $this->offset_bottom;
 		$this->canvas_y = $this->offset_top;
 
-		// Set missing properties for left Y axis.
-		if ($this->left_y_min === null) {
-			$this->left_y_min = $this->min_value_left ? : 0;
-		}
-		if ($this->left_y_max === null) {
-			$this->left_y_max = $this->max_value_left ? : 1;
-		}
-
-		if (bccomp($this->left_y_min, $this->left_y_max) == 0) {
-			$this->left_y_min -= 0.5;
-			$this->left_y_max += 0.5;
-		}
-		elseif (bccomp($this->left_y_min, $this->left_y_max) == 1) {
-			$this->left_y_max = $this->left_y_min + 1;
-		}
-
-		$grid = $this->getValueGrid($this->left_y_min, $this->left_y_max);
-		$this->left_y_min = $grid[0];
-		$this->left_y_max = end($grid);
+		// Determine units for left side.
 
 		if ($this->left_y_units === null) {
 			$this->left_y_units = '';
@@ -529,25 +527,7 @@ class CSvgGraph extends CSvg {
 			}
 		}
 
-		// Set missing properties for right Y axis.
-		if ($this->right_y_min === null) {
-			$this->right_y_min = $this->min_value_right ? : 0;
-		}
-		if ($this->right_y_max === null) {
-			$this->right_y_max = $this->max_value_right ? : 1;
-		}
-
-		if (bccomp($this->right_y_min, $this->right_y_max) == 0) {
-			$this->right_y_min -= 0.5;
-			$this->right_y_max += 0.5;
-		}
-		elseif (bccomp($this->right_y_min, $this->right_y_max) == 1) {
-			$this->right_y_max = $this->right_y_min + 1;
-		}
-
-		$grid = $this->getValueGrid($this->right_y_min, $this->right_y_max);
-		$this->right_y_min = $grid[0];
-		$this->right_y_max = end($grid);
+		// Determine units for right side.
 
 		if ($this->right_y_units === null) {
 			$this->right_y_units = '';
@@ -559,7 +539,65 @@ class CSvgGraph extends CSvg {
 			}
 		}
 
+		// Calculate vertical scale parameters for left side.
+
+		$rows_min = (int) max(1, floor($this->canvas_height / $this->cell_height_min / 1.5));
+		$rows_max = (int) max(1, floor($this->canvas_height / $this->cell_height_min));
+
+		$this->left_y_min_calculated = $this->left_y_min === null;
+		$this->left_y_max_calculated = $this->left_y_max === null;
+
+		if ($this->left_y_min_calculated) {
+			$this->left_y_min = $this->min_value_left ? : 0;
+		}
+		if ($this->left_y_max_calculated) {
+			$this->left_y_max = $this->max_value_left ? : 1;
+		}
+
+		$this->left_y_is_binary = $this->left_y_units === 'B' || $this->left_y_units === 'Bps';
+
+		$result = calculateGraphScaleExtremes($this->left_y_min, $this->left_y_max, $this->left_y_is_binary,
+			$this->left_y_min_calculated, $this->left_y_max_calculated, $rows_min, $rows_max
+		);
+
+		[
+			'min' => $this->left_y_min,
+			'max' => $this->left_y_max,
+			'interval' => $this->left_y_interval,
+			'power' => $this->left_y_power
+		] = $result;
+
+		// Calculate vertical scale parameters for right side.
+
+		if ($this->left_y_min_calculated && $this->left_y_max_calculated) {
+			$rows_min = $rows_max = $result['rows'];
+		}
+
+		$this->right_y_min_calculated = $this->right_y_min === null;
+		$this->right_y_max_calculated = $this->right_y_max === null;
+
+		if ($this->right_y_min_calculated) {
+			$this->right_y_min = $this->min_value_right ? : 0;
+		}
+		if ($this->right_y_max_calculated) {
+			$this->right_y_max = $this->max_value_right ? : 1;
+		}
+
+		$this->right_y_is_binary = $this->right_y_units === 'B' || $this->right_y_units === 'Bps';
+
+		$result = calculateGraphScaleExtremes($this->right_y_min, $this->right_y_max, $this->right_y_is_binary,
+			$this->right_y_min_calculated, $this->right_y_max_calculated, $rows_min, $rows_max
+		);
+
+		[
+			'min' => $this->right_y_min,
+			'max' => $this->right_y_max,
+			'interval' => $this->right_y_interval,
+			'power' => $this->right_y_power
+		] = $result;
+
 		// Define canvas dimensions and offsets, except canvas height and bottom offset.
+
 		$approx_width = 10;
 
 		if ($this->left_y_show) {
@@ -584,58 +622,80 @@ class CSvgGraph extends CSvg {
 		$this->canvas_width = $this->width - $this->offset_left - $this->offset_right;
 		$this->canvas_x = $this->offset_left;
 
-		// Calculate Y = 0 position.
-		$delta = (($this->right_y_max - $this->right_y_min) ? : 1);
-		$this->right_y_zero = $this->canvas_y + $this->canvas_height * $this->right_y_max / $delta;
+		// Calculate vertical zero position.
 
-		$delta = (($this->left_y_max - $this->left_y_min) ? : 1);
-		$this->left_y_zero = $this->canvas_y + $this->canvas_height * $this->left_y_max / $delta;
+		if ($this->left_y_max - $this->left_y_min == INF) {
+			$this->left_y_zero = $this->canvas_y + $this->canvas_height
+				* max(0, min(1, $this->left_y_max / 10 / ($this->left_y_max / 10 - $this->left_y_min / 10)));
+		}
+		else {
+			$this->left_y_zero = $this->canvas_y + $this->canvas_height
+				* max(0, min(1, $this->left_y_max / ($this->left_y_max - $this->left_y_min)));
+		}
+
+		if ($this->right_y_max - $this->right_y_min == INF) {
+			$this->right_y_zero = $this->canvas_y + $this->canvas_height
+				* max(0, min(1, $this->right_y_max / 10 / ($this->right_y_max / 10 - $this->right_y_min / 10)));
+		}
+		else {
+			$this->right_y_zero = $this->canvas_y + $this->canvas_height
+				* max(0, min(1, $this->right_y_max / ($this->right_y_max - $this->right_y_min)));
+		}
 	}
 
 	/**
 	 * Get array of X points with labels, for grid and X/Y axes. Array key is Y coordinate for SVG, value is label with
 	 * axis units.
 	 *
-	 * @param int $side  Type of X axis: GRAPH_YAXIS_SIDE_RIGHT, GRAPH_YAXIS_SIDE_LEFT
+	 * @param int  $side       Type of Y axis: GRAPH_YAXIS_SIDE_RIGHT, GRAPH_YAXIS_SIDE_LEFT
+	 * @param bool $empty_set  Return defaults for empty side.
 	 *
 	 * @return array
 	 */
 	protected function getValuesGridWithPosition($side, $empty_set = false) {
-		if ($empty_set) {
-			$units = '';
-		}
-		elseif ($side === GRAPH_YAXIS_SIDE_LEFT) {
-			$min_value = $this->left_y_min;
-			$max_value = $this->left_y_max;
-			$units = $this->left_y_units;
-		}
-		else {
-			$min_value = $this->right_y_min;
-			$max_value = $this->right_y_max;
-			$units = $this->right_y_units;
-		}
+		$min = 0;
+		$max = 1;
+		$min_calculated = true;
+		$max_calculated = true;
+		$interval = 1;
+		$units = '';
+		$is_binary = false;
+		$power = 0;
 
-		$grid = $empty_set ? [0, 1] : $this->getValueGrid($min_value, $max_value);
-		$min_value = $grid[0];
-		$max_value = end($grid);
-		$delta = ($max_value != $min_value)
-			? $max_value - $min_value
-			: (count($grid) > 1 ? end($grid) - $grid[0] : 1);
-		$grid_values = [];
-
-		foreach ($grid as $value) {
-			$relative_pos = $this->canvas_height - intval($this->canvas_height * ($max_value - $value) / $delta);
-
-			if ($relative_pos >= 0 && $relative_pos <= $this->canvas_height) {
-				$grid_values[$relative_pos] = convert_units([
-					'value' => $value,
-					'units' => $units,
-					'convert' => ITEM_CONVERT_NO_UNITS
-				]);
+		if (!$empty_set) {
+			if ($side === GRAPH_YAXIS_SIDE_LEFT) {
+				$min = $this->left_y_min;
+				$max = $this->left_y_max;
+				$min_calculated = $this->left_y_min_calculated;
+				$max_calculated = $this->left_y_max_calculated;
+				$interval = $this->left_y_interval;
+				$units = $this->left_y_units;
+				$is_binary = $this->left_y_is_binary;
+				$power = $this->left_y_power;
+			}
+			elseif ($side === GRAPH_YAXIS_SIDE_RIGHT) {
+				$min = $this->right_y_min;
+				$max = $this->right_y_max;
+				$min_calculated = $this->right_y_min_calculated;
+				$max_calculated = $this->right_y_max_calculated;
+				$interval = $this->right_y_interval;
+				$units = $this->right_y_units;
+				$is_binary = $this->right_y_is_binary;
+				$power = $this->right_y_power;
 			}
 		}
 
-		return $grid_values;
+		$relative_values = calculateGraphScaleValues($min, $max, $min_calculated, $max_calculated, $interval, $units,
+			$is_binary, $power, 14
+		);
+
+		$absolute_values = [];
+
+		foreach ($relative_values as ['relative_pos' => $relative_pos, 'value' => $value]) {
+			$absolute_values[(int) round($this->canvas_height * $relative_pos)] = $value;
+		}
+
+		return $absolute_values;
 	}
 
 	/**
@@ -645,10 +705,10 @@ class CSvgGraph extends CSvg {
 		$grid_values = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_LEFT, $this->left_y_empty);
 		$this->addItem(
 			(new CSvgGraphAxis($grid_values, GRAPH_YAXIS_SIDE_LEFT))
+				->setLineColor($this->grid_color)
+				->setTextColor($this->text_color)
 				->setSize($this->offset_left, $this->canvas_height)
 				->setPosition($this->canvas_x - $this->offset_left, $this->canvas_y)
-				->setTextColor($this->text_color)
-				->setLineColor($this->grid_color)
 		);
 	}
 
@@ -665,10 +725,10 @@ class CSvgGraph extends CSvg {
 
 		$this->addItem(
 			(new CSvgGraphAxis($grid_values, GRAPH_YAXIS_SIDE_RIGHT))
+				->setLineColor($this->grid_color)
+				->setTextColor($this->text_color)
 				->setSize($this->offset_right, $this->canvas_height)
 				->setPosition($this->canvas_x + $this->canvas_width, $this->canvas_y)
-				->setTextColor($this->text_color)
-				->setLineColor($this->grid_color)
 		);
 	}
 
@@ -678,52 +738,13 @@ class CSvgGraph extends CSvg {
 	protected function drawCanvasXAxis() {
 		$this->addItem(
 			(new CSvgGraphAxis($this->getTimeGridWithPosition(), GRAPH_YAXIS_SIDE_BOTTOM))
+				->setLineColor($this->grid_color)
+				->setTextColor($this->text_color)
 				->setSize($this->canvas_width, $this->xaxis_height)
 				->setPosition($this->canvas_x, $this->canvas_y + $this->canvas_height)
-				->setTextColor($this->text_color)
-				->setLineColor($this->grid_color)
 		);
 	}
 
-	/**
-	 * Calculate array of points between $min and $max value.
-	 *
-	 * @param int $min  Minimum value.
-	 * @param int $max  Maximum value.
-	 *
-	 * @return $array
-	 */
-	protected function getValueGrid($min, $max) {
-		$res = [];
-
-		// If absolute min/max is equal, calculate grid with 4 rows to make 0 centered. 5 rows otherwise.
-		$grid_rows = (abs($min) == abs($max)) ? 4 : 5;
-		$decimals = strlen(substr(strrchr($max, '.'), 1));
-		$decimals = $decimals > 4 ? 4 : $decimals;
-		$decimals = $decimals < 2 ? 2 : $decimals;
-		for ($base = 10; $base > .01; $base /= 10) {
-			$mul = ($max > 0) ? 1 / pow($base, floor(log10($max))) : 1;
-			$max10 = ceil($mul * $max) / $mul;
-			$min10 = floor($mul * $min) / $mul;
-			$delta = $max10 - $min10;
-			$delta = ceil($mul * $delta) / $mul;
-
-			if ($mul >= 1) {
-				if ($delta) {
-					for ($i = 0; $delta >= $i; $i += $delta / $grid_rows) {
-						$res[] = sprintf('%.'.$decimals.'f', $i + $min10);
-					}
-				}
-				else {
-					$res[] = $min10;
-				}
-				break;
-			}
-		}
-
-		return $res;
-	}
-
 	/**
 	 * Add grid to graph.
 	 */
@@ -747,9 +768,9 @@ class CSvgGraph extends CSvg {
 		}
 
 		$this->addItem((new CSvgGraphGrid($value_points, $time_points))
+			->setColor($this->grid_color)
 			->setPosition($this->canvas_x, $this->canvas_y)
 			->setSize($this->canvas_width, $this->canvas_height)
-			->setColor($this->grid_color)
 		);
 	}
 
@@ -759,7 +780,7 @@ class CSvgGraph extends CSvg {
 	protected function calculatePaths() {
 		// Metric having very big values of y outside visible area will fail to render.
 		$y_max = pow(2, 16);
-		$y_min = $y_max * -1;
+		$y_min = -$y_max;
 
 		foreach ($this->metrics as $index => $metric) {
 			if (!array_key_exists($index, $this->points)) {
@@ -776,7 +797,6 @@ class CSvgGraph extends CSvg {
 			}
 
 			$time_range = ($this->time_till - $this->time_from) ? : 1;
-			$value_diff = ($max_value - $min_value) ? : 1;
 			$timeshift = $metric['options']['timeshift'];
 			$paths = [];
 
@@ -796,16 +816,35 @@ class CSvgGraph extends CSvg {
 				if ($in_range || $metric['options']['type'] != SVG_GRAPH_TYPE_POINTS) {
 					$x = $this->canvas_x + $this->canvas_width
 						- $this->canvas_width * ($this->time_till - $clock + $timeshift) / $time_range;
-					$y = $this->canvas_y + $this->canvas_height * ($max_value - $point) / $value_diff;
+
+					if ($max_value - $min_value == INF) {
+						$y = $this->canvas_y + CMathHelper::safeMul([$this->canvas_height,
+							$max_value / 10 - $point / 10, 1 / ($max_value / 10 - $min_value / 10)
+						]);
+					}
+					else {
+						$y = $this->canvas_y + CMathHelper::safeMul([$this->canvas_height,
+							$max_value - $point, 1 / ($max_value - $min_value)
+						]);
+					}
 
 					if (!$in_range) {
 						$y = ($point > $max_value) ? max($y_min, $y) : min($y_max, $y);
 					}
 
-					$paths[$path_num][] = [$x, ceil($y), convert_units([
-						'value' => $point,
-						'units' => $metric['units']
-					])];
+					if ($metric['options']['type'] == SVG_GRAPH_TYPE_BAR
+							|| $metric['options']['type'] == SVG_GRAPH_TYPE_POINTS) {
+						$paths[] = [$x, ceil($y), convertUnits([
+							'value' => $point,
+							'units' => $metric['units']
+						])];
+					}
+					else {
+						$paths[$path_num][] = [$x, ceil($y), convertUnits([
+							'value' => $point,
+							'units' => $metric['units']
+						])];
+					}
 				}
 			}
 
@@ -815,93 +854,6 @@ class CSvgGraph extends CSvg {
 		}
 	}
 
-	/**
-	 * Modifies metric data and Y value range according specified missing data function.
-	 */
-	protected function applyMissingDataFunc() {
-		foreach ($this->metrics as $index => $metric) {
-			/**
-			 * - Missing data points are calculated only between existing data points;
-			 * - Missing data points are not calculated for SVG_GRAPH_TYPE_POINTS && SVG_GRAPH_TYPE_BAR metrics;
-			 * - SVG_GRAPH_MISSING_DATA_CONNECTED is default behavior of SVG graphs, so no need to calculate anything
-			 *   here.
-			 */
-			if (array_key_exists($index, $this->points)
-					&& !in_array($metric['options']['type'], [SVG_GRAPH_TYPE_POINTS, SVG_GRAPH_TYPE_BAR])
-					&& $metric['options']['missingdatafunc'] != SVG_GRAPH_MISSING_DATA_CONNECTED) {
-				$points = &$this->points[$index];
-				$missing_data_points = $this->getMissingData($points, $metric['options']['missingdatafunc']);
-
-				// Sort according new clock times (array keys).
-				$points += $missing_data_points;
-				ksort($points);
-
-				// Missing data function can change min value of Y axis.
-				if ($missing_data_points
-						&& $metric['options']['missingdatafunc'] == SVG_GRAPH_MISSING_DATA_TREAT_AS_ZERO) {
-					if ($this->min_value_left > 0 && $metric['options']['axisy'] == GRAPH_YAXIS_SIDE_LEFT) {
-						$this->min_value_left = 0;
-					}
-					elseif ($this->min_value_right > 0 && $metric['options']['axisy'] == GRAPH_YAXIS_SIDE_RIGHT) {
-						$this->min_value_right = 0;
-					}
-				}
-			}
-		}
-	}
-
-	/**
-	 * Calculate missing data for given set of $points according given $missingdatafunc.
-	 *
-	 * @param array $points           Array of metric points to modify, where key is metric timestamp.
-	 * @param int   $missingdatafunc  Type of function, allowed value:
-	 *                                SVG_GRAPH_MISSING_DATA_TREAT_AS_ZERO, SVG_GRAPH_MISSING_DATA_NONE,
-	 *                                SVG_GRAPH_MISSING_DATA_CONNECTED
-	 *
-	 * @return array  Array of calculated missing data points.
-	 */
-	protected function getMissingData(array $points = [], $missingdatafunc) {
-		// Get average distance between points to detect gaps of missing data.
-		$prev_clock = null;
-		$points_distance = [];
-		foreach ($points as $clock => $point) {
-			if ($prev_clock !== null) {
-				$points_distance[] = $clock - $prev_clock;
-			}
-			$prev_clock = $clock;
-		}
-
-		/**
-		 * $threshold          is a minimal period of time at what we assume that data point is missed;
-		 * $average_distance   is an average distance between existing data points;
-		 * $gap_interval       is a time distance between missing points used to fulfill gaps of missing data.
-		 *                     It's unique for each gap.
-		 */
-		$average_distance = $points_distance ? array_sum($points_distance) / count($points_distance) : 0;
-		$threshold = $points_distance ? $average_distance * 3 : 0;
-
-		// Add missing values.
-		$prev_clock = null;
-		$missing_points = [];
-		foreach ($points as $clock => $point) {
-			if ($prev_clock !== null && ($clock - $prev_clock) > $threshold) {
-				$gap_interval = floor(($clock - $prev_clock) / $threshold);
-
-				if ($missingdatafunc == SVG_GRAPH_MISSING_DATA_NONE) {
-					$missing_points[$prev_clock + $gap_interval] = null;
-				}
-				elseif ($missingdatafunc == SVG_GRAPH_MISSING_DATA_TREAT_AS_ZERO) {
-					$missing_points[$prev_clock + $gap_interval] = 0;
-					$missing_points[$clock - $gap_interval] = 0;
-				}
-			}
-
-			$prev_clock = $clock;
-		}
-
-		return $missing_points;
-	}
-
 	/**
 	 * Add fill area to graph for metric of type SVG_GRAPH_TYPE_LINE or SVG_GRAPH_TYPE_STAIRCASE.
 	 */
@@ -923,21 +875,139 @@ class CSvgGraph extends CSvg {
 			if (array_key_exists($index, $this->paths) && ($metric['options']['type'] == SVG_GRAPH_TYPE_LINE
 					|| $metric['options']['type'] == SVG_GRAPH_TYPE_STAIRCASE)) {
 				if ($metric['options']['fill'] > 0) {
-					$this->drawMetricArea($metric, $this->paths[$index]);
+					$paths = $this->decoratePathVirualArea($this->paths[$index], $metric);
+					$this->drawMetricArea($metric, $paths);
 				}
 
-				$this->addItem(new CSvgGraphLineGroup($this->paths[$index], $metric));
+				$paths = $this->decoratePathVirualLine($this->paths[$index], $metric);
+				$this->addItem(new CSvgGraphLineGroup($paths, $metric));
 			}
 		}
 	}
 
+	/**
+	 * Edit the first and last path segments to stretch up till edge of graph.
+	 *
+	 * @param array $paths
+	 * @param array $metric
+	 *
+	 * @return array  Modified paths.
+	 */
+	protected function decoratePathVirualArea(array $paths, array $metric): array {
+		$connect_left = false;
+		$connect_right = false;
+
+		$timestamps = array_keys($this->points[$metric['options']['order']]);
+
+		$first_path =& $paths[0];
+		$first_point =& $first_path[0];
+		$first_point_time = reset($timestamps);
+
+		$last_path =& $paths[count($paths) - 1];
+		$last_point =& $last_path[count($last_path) - 1];
+		$last_point_time = end($timestamps);
+
+		if ($metric['options']['frequency'] <= 0) {
+			$connect_left = true;
+			$connect_right = true;
+		}
+		else {
+			$diff_left = $first_point_time - $this->time_from;
+			$connect_left = $diff_left < 3 * $metric['options']['frequency'];
+
+			$diff_right = min($this->time_till, time()) - $last_point_time;
+			$connect_right = $diff_right < 3 * $metric['options']['frequency'];
+		}
+
+		if ($connect_left) {
+			if (count($paths) > 1) {
+				$first_point[0] = 0;
+			}
+			else {
+				array_unshift($first_path, [0 => 0] + $first_point);
+			}
+		}
+
+		if ($connect_right) {
+			if ($this->time_till > time()) {
+				$seconds_in_pixel = ($this->time_till - $this->time_from) / $this->width;
+				$right_edge = (time() - $this->time_from) / $seconds_in_pixel;
+			}
+			else {
+				$right_edge = $this->width;
+			}
+
+			if (count($paths) > 1) {
+				$last_point[0] = $right_edge;
+			}
+			else {
+				$last_path[] = [0 => $right_edge] + $last_point;
+			}
+		}
+
+		return $paths;
+	}
+
+	/**
+	 * Extend first and last path segments till edge of graph.
+	 *
+	 * @param array $paths
+	 * @param array $metric
+	 *
+	 * @return array  Modified paths.
+	 */
+	protected function decoratePathVirualLine(array $paths, array $metric): array {
+		$connect_left = false;
+		$connect_right = false;
+
+		$timestamps = array_keys($this->points[$metric['options']['order']]);
+
+		$first_path =& $paths[0];
+		$first_point = $first_path[0];
+		$first_point_time = reset($timestamps);
+
+		$last_path =& $paths[count($paths) - 1];
+		$last_point = $last_path[count($last_path) - 1];
+		$last_point_time = end($timestamps);
+
+		if ($metric['options']['frequency'] <= 0) {
+			$connect_left = true;
+			$connect_right = true;
+		}
+		else {
+			$diff_left = $first_point_time - $this->time_from;
+			$connect_left = $diff_left < 3 * $metric['options']['frequency'];
+
+			$diff_right = min($this->time_till, time()) - $last_point_time;
+			$connect_right = $diff_right < 3 * $metric['options']['frequency'];
+		}
+
+		if ($connect_left) {
+			array_unshift($first_path, [0 => 0] + $first_point);
+		}
+
+		if ($connect_right) {
+			if ($this->time_till > time()) {
+				$seconds_in_pixel = ($this->time_till - $this->time_from) / $this->width;
+				$right_edge = (time() - $this->time_from) / $seconds_in_pixel;
+			}
+			else {
+				$right_edge = $this->width;
+			}
+
+			$last_path[] = [0 => $right_edge] + $last_point;
+		}
+
+		return $paths;
+	}
+
 	/**
 	 * Add metric of type points to graph.
 	 */
 	protected function drawMetricsPoint() {
 		foreach ($this->metrics as $index => $metric) {
 			if ($metric['options']['type'] == SVG_GRAPH_TYPE_POINTS && array_key_exists($index, $this->paths)) {
-				$this->addItem(new CSvgGraphPoints(reset($this->paths[$index]), $metric));
+				$this->addItem(new CSvgGraphPoints($this->paths[$index], $metric));
 			}
 		}
 	}
@@ -962,7 +1032,6 @@ class CSvgGraph extends CSvg {
 				$y_axis_side = $this->metrics[$index]['options']['axisy'];
 				$time_points = array_keys($this->points[$index]);
 				$last_point = 0;
-				$path = reset($path);
 
 				foreach ($path as $point_index => $point) {
 					$time_point = ($sec_per_px > $px_per_sec)
@@ -988,16 +1057,16 @@ class CSvgGraph extends CSvg {
 				foreach ($paths as $path_index => $point_index) {
 					$group_x = $bar_groups_position[$y_axis][$time_point][$point_index];
 					if ($group_count > 1) {
-						$this->paths[$path_index][0][$point_index][0] = $group_x
+						$this->paths[$path_index][$point_index][0] = $group_x
 							// Calculate the leftmost X-coordinate including gap size.
 							- $group_width * .375
 							// Calculate the X-offset for the each bar in the group.
 							+ ceil($bar_width * ($group_index + .5));
 						$group_index++;
 					}
-					$this->paths[$path_index][0][$point_index][3] = max(1, $bar_width);
+					$this->paths[$path_index][$point_index][3] = max(1, $bar_width);
 					// X position for bars group.
-					$this->paths[$path_index][0][$point_index][4] = $group_x - $group_width * .375;
+					$this->paths[$path_index][$point_index][4] = $group_x - $group_width * .375;
 				}
 			}
 		}
@@ -1009,7 +1078,7 @@ class CSvgGraph extends CSvg {
 					: $this->left_y_zero;
 				$metric['options']['bar_width'] = $bar_min_width[$metric['options']['axisy']];
 
-				$this->addItem(new CSvgGraphBar(reset($this->paths[$index]), $metric));
+				$this->addItem(new CSvgGraphBar($this->paths[$index], $metric));
 			}
 		}
 	}
@@ -1088,7 +1157,7 @@ class CSvgGraph extends CSvg {
 
 			$container->addItem(
 				(new CSvgGraphAnnotation($draw_type))
-					->setInformation(CJs::encodeJson($info))
+					->setInformation(json_encode($info))
 					->setSize(min($x2 - $x1, $this->canvas_width), $this->canvas_height)
 					->setPosition(max($x1, $this->canvas_x), $this->canvas_y)
 					->setColor($this->color_annotation)
