@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -71,7 +71,7 @@ class CConfigurationImport {
 			'templates' => ['updateExisting' => false, 'createMissing' => false],
 			'templateScreens' => ['updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false],
 			'applications' => ['createMissing' => false, 'deleteMissing' => false],
-			'templateLinkage' => ['createMissing' => false],
+			'templateLinkage' => ['createMissing' => false, 'deleteMissing' => false],
 			'items' => ['updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false],
 			'discoveryRules' => ['updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false],
 			'triggers' => ['updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false],
@@ -88,6 +88,7 @@ class CConfigurationImport {
 
 		$object_options = (
 			$options['templateLinkage']['createMissing']
+			|| $options['templateLinkage']['deleteMissing']
 			|| $options['applications']['createMissing']
 			|| $options['applications']['deleteMissing']
 			|| $options['items']['updateExisting']
@@ -140,7 +141,6 @@ class CConfigurationImport {
 		$this->deleteMissingTriggers();
 		$this->deleteMissingGraphs();
 		$this->deleteMissingItems();
-		$this->deleteMissingApplications();
 
 		// import objects
 		$this->processApplications();
@@ -156,6 +156,9 @@ class CConfigurationImport {
 		$this->processScreens();
 		$this->processMediaTypes();
 
+		// Missing applications should be deleted after new application changes are done to all inherited items.
+		$this->deleteMissingApplications();
+
 		return true;
 	}
 
@@ -321,6 +324,19 @@ class CConfigurationImport {
 						$templatesRefs[$template['name']] = $template['name'];
 					}
 				}
+
+				if ($discoveryRule['overrides']) {
+					foreach ($discoveryRule['overrides'] as $override) {
+						foreach ($override['operations'] as $operation) {
+							if ($operation['operationobject'] == OPERATION_OBJECT_HOST_PROTOTYPE
+									&& array_key_exists('optemplate', $operation)) {
+								foreach ($operation['optemplate'] as $template) {
+									$templatesRefs[$template['name']] = $template['name'];
+								}
+							}
+						}
+					}
+				}
 			}
 		}
 
@@ -976,13 +992,42 @@ class CConfigurationImport {
 
 				unset($item['master_item']);
 
+				if ($item['overrides']) {
+					foreach ($item['overrides'] as &$override) {
+						foreach ($override['operations'] as &$operation) {
+							if ($operation['operationobject'] == OPERATION_OBJECT_HOST_PROTOTYPE
+									&& array_key_exists('optemplate', $operation)) {
+								foreach ($operation['optemplate'] as &$template) {
+									$templateid = $this->referencer->resolveTemplate($template['name']);
+
+									if (!$templateid) {
+										throw new Exception(_s(
+											'Cannot find template "%1$s" for override "%2$s" of discovery rule "%3$s" on "%4$s".',
+											$template['name'],
+											$override['name'],
+											$item['name'],
+											$host
+										));
+									}
+
+									$template['templateid'] = $templateid;
+
+									unset($template['name']);
+								}
+							}
+						}
+						unset($operation);
+					}
+					unset($override);
+				}
+
 				if ($itemId) {
 					$item['itemid'] = $itemId;
 					$itemsToUpdate[] = $item;
 				}
 				else {
 					/*
-					 * The array key "lld_macro_paths" must exist at this point. It is processed by chain convertion.
+					 * The array key "lld_macro_paths" must exist at this point. It is processed by chain conversion.
 					 * Unlike discoveryrule.update method, discoveryrule.create does not allow "lld_macro_paths"
 					 * to be empty.
 					 */
