@@ -211,8 +211,7 @@ class CWidgetHelper {
 	 */
 	private static function getMultiselectField($field, $captions, $form_name, $object_name, $popup_options) {
 		$field_name = $field->getName().($field->isMultiple() ? '[]' : '');
-
-		return (new CMultiSelect([
+		$options = [
 			'name' => $field_name,
 			'object_name' => $object_name,
 			'multiple' => $field->isMultiple(),
@@ -224,7 +223,13 @@ class CWidgetHelper {
 				] + $popup_options
 			],
 			'add_post_js' => false
-		]))
+		];
+
+		if ($field instanceof CWidgetFieldMsHost && $field->filter_preselect_host_group_field) {
+			$options['popup']['filter_preselect_fields']['hostgroups'] = $field->filter_preselect_host_group_field;
+		}
+
+		return (new CMultiSelect($options))
 			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 			->setAriaRequired(self::isAriaRequired($field));
 	}
@@ -333,7 +338,7 @@ class CWidgetHelper {
 			(new CButton('select', _('Select')))
 				->addClass(ZBX_STYLE_BTN_GREY)
 				->onClick('return PopUp("popup.generic",'.
-					CJs::encodeJson($field->getPopupOptions($form_name)).', null, this);')
+					json_encode($field->getPopupOptions($form_name)).', null, this);')
 		];
 	}
 
@@ -395,34 +400,26 @@ class CWidgetHelper {
 
 	/**
 	 * @param CWidgetFieldSeverities $field
-	 * @param array $config
 	 *
-	 * @return CList
+	 * @return CSeverityCheckBoxList
 	 */
-	public static function getSeverities($field, $config) {
-		$severities = [];
-
-		for ($severity = TRIGGER_SEVERITY_NOT_CLASSIFIED; $severity < TRIGGER_SEVERITY_COUNT; $severity++) {
-			$severities[$severity] = getSeverityName($severity, $config);
-		}
-
-		return self::getCheckBoxList($field, $severities);
+	public static function getSeverities($field) {
+		return (new CSeverityCheckBoxList($field->getName()))
+			->setChecked($field->getValue())
+			->setEnabled(!($field->getFlags() & CWidgetField::FLAG_DISABLED))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
 	}
 
 	/**
 	 * @param CWidgetFieldCheckBoxList $field
-	 * @param array $config
+	 * @param array                    $list  Option list array.
 	 *
 	 * @return CList
 	 */
-	public static function getCheckBoxList($field, array $config) {
+	public static function getCheckBoxList($field, array $list) {
 		$checkbox_list = (new CList())->addClass(ZBX_STYLE_LIST_CHECK_RADIO);
 
-		if ($field->getOrientation() == CWidgetFieldCheckBoxList::ORIENTATION_HORIZONTAL) {
-			$checkbox_list->addClass(ZBX_STYLE_HOR_LIST);
-		}
-
-		foreach ($config as $key => $label) {
+		foreach ($list as $key => $label) {
 			$checkbox_list->addItem(
 				(new CCheckBox($field->getName().'[]', $key))
 					->setLabel($label)
@@ -539,6 +536,13 @@ class CWidgetHelper {
 	 * @return array
 	 */
 	public static function getApplicationSelector($field) {
+		$popup_options = json_encode($field->getFilterParameters());
+
+		if ($field->filter_preselect_host_field) {
+			$popup_options = 'jQuery.extend('.
+				$popup_options.', getFirstMultiselectValue("'.$field->filter_preselect_host_field.'"))';
+		}
+
 		return [
 			(new CTextBox($field->getName(), $field->getValue()))
 				->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
@@ -547,9 +551,7 @@ class CWidgetHelper {
 			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 			(new CButton($field->getName().'_select', _('Select')))
 				->addClass(ZBX_STYLE_BTN_GREY)
-				->onClick(
-					'return PopUp("popup.generic", '.CJs::encodeJson($field->getFilterParameters()).', null, this);'
-				)
+				->onClick('return PopUp("popup.generic", '.$popup_options.', null, this);')
 		];
 	}
 
@@ -631,8 +633,9 @@ class CWidgetHelper {
 							->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
 					))->addClass(ZBX_STYLE_COLUMN_50)
 				]))
-					->addClass(ZBX_STYLE_COLUMN_95)
-					->addClass(ZBX_STYLE_COLUMNS),
+					->addClass(ZBX_STYLE_COLUMNS)
+					->addClass(ZBX_STYLE_COLUMNS_NOWRAP)
+					->addClass(ZBX_STYLE_COLUMN_95),
 
 				(new CDiv(
 					(new CButton())
@@ -827,7 +830,7 @@ class CWidgetHelper {
 				'jQuery("#overrides .'.ZBX_STYLE_OVERRIDES_OPTIONS_LIST.'").overrides({'.
 					'add: ".'.ZBX_STYLE_BTN_ALT.'",'.
 					'options: "input[type=hidden]",'.
-					'captions: '.CJs::encodeJson(self::getGraphOverrideOptionNames()).','.
+					'captions: '.json_encode(self::getGraphOverrideOptionNames()).','.
 					'makeName: function(option, row_id) {'.
 						'return "'.$field->getName().'[" + row_id + "][" + option + "]";'.
 					'},'.
@@ -839,7 +842,7 @@ class CWidgetHelper {
 					'override: ".'.ZBX_STYLE_OVERRIDES_LIST_ITEM.'",'.
 					'overridesList: ".'.ZBX_STYLE_OVERRIDES_LIST.'",'.
 					'onUpdate: onGraphConfigChange,'.
-					'menu: '.CJs::encodeJson(self::getGraphOverrideMenu()).
+					'menu: '.json_encode(self::getGraphOverrideMenu()).
 				'});'.
 			'}',
 
@@ -854,7 +857,9 @@ class CWidgetHelper {
 				'})'.
 				'.bind("afteradd.dynamicRows", function(event, options) {'.
 					'var container = jQuery(".overlay-dialogue-body");'.
-					'container.scrollTop(container[0].scrollHeight);'.
+					'container.scrollTop(Math.max(container.scrollTop(),
+						jQuery("#widget_dialogue_form")[0].scrollHeight - container.height()
+					));'.
 
 					'jQuery(".multiselect", jQuery("#overrides")).each(function() {'.
 						'jQuery(this).multiSelect(jQuery(this).data("params"));'.
@@ -897,7 +902,7 @@ class CWidgetHelper {
 				'handle: ".drag-icon",'.
 				'tolerance: "pointer",'.
 				'scroll: false,'.
-				'cursor: IE ? "move" : "grabbing",'.
+				'cursor: "grabbing",'.
 				'opacity: 0.6,'.
 				'axis: "y",'.
 				'disabled: function() {'.
@@ -981,8 +986,9 @@ class CWidgetHelper {
 						]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
 					))->addClass(ZBX_STYLE_COLUMN_50),
 				]))
-					->addClass(ZBX_STYLE_COLUMN_95)
-					->addClass(ZBX_STYLE_COLUMNS),
+					->addClass(ZBX_STYLE_COLUMNS)
+					->addClass(ZBX_STYLE_COLUMNS_NOWRAP)
+					->addClass(ZBX_STYLE_COLUMN_95),
 				(new CDiv([
 					(new CButton())
 						->setAttribute('title', _('Delete'))
@@ -1118,6 +1124,7 @@ class CWidgetHelper {
 						->addClass(ZBX_STYLE_COLUMN_50),
 				]))
 					->addClass(ZBX_STYLE_COLUMNS)
+					->addClass(ZBX_STYLE_COLUMNS_NOWRAP)
 					->addClass(ZBX_STYLE_COLUMN_95)
 			))
 				->addClass(ZBX_STYLE_LIST_ACCORDION_ITEM_BODY)
@@ -1243,12 +1250,14 @@ class CWidgetHelper {
 				'})'.
 				'.bind("afteradd.dynamicRows", function(event, options) {'.
 					'var container = jQuery(".overlay-dialogue-body");'.
-					'container.scrollTop(container[0].scrollHeight);'.
+					'container.scrollTop(Math.max(container.scrollTop(),
+						jQuery("#widget_dialogue_form")[0].scrollHeight - container.height()
+					));'.
 
 					'jQuery(".input-color-picker input").colorpicker({onUpdate: function(color) {'.
 						'var ds = jQuery(this).closest(".'.ZBX_STYLE_LIST_ACCORDION_ITEM.'");'.
 						'jQuery(".'.ZBX_STYLE_COLOR_PREVIEW_BOX.'", ds).css("background-color", "#"+color);'.
-					'}, appendTo: "#overlay_dialogue"});'.
+					'}, appendTo: ".overlay-dialogue-body"});'.
 
 					'jQuery(".multiselect", jQuery("#data_sets")).each(function() {'.
 						'jQuery(this).multiSelect(jQuery(this).data("params"));'.
@@ -1310,7 +1319,7 @@ class CWidgetHelper {
 			'jQuery(".input-color-picker input").colorpicker({onUpdate: function(color){'.
 				'var ds = jQuery(this).closest(".'.ZBX_STYLE_LIST_ACCORDION_ITEM.'");'.
 				'jQuery(".'.ZBX_STYLE_COLOR_PREVIEW_BOX.'", ds).css("background-color", "#"+color);'.
-			'}, appendTo: "#overlay_dialogue"});',
+			'}, appendTo: ".overlay-dialogue-body"});',
 
 			// Initialize sortability.
 			'if (jQuery("#data_sets .'.ZBX_STYLE_LIST_ACCORDION_ITEM.'").length < 2) {'.
@@ -1322,7 +1331,7 @@ class CWidgetHelper {
 				'handle: ".drag-icon",'.
 				'tolerance: "pointer",'.
 				'scroll: false,'.
-				'cursor: IE ? "move" : "grabbing",'.
+				'cursor: "grabbing",'.
 				'opacity: 0.6,'.
 				'axis: "y",'.
 				'disabled: function() {'.
