@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -157,31 +157,21 @@ function get_icon($type, $params = []) {
 
 			return $icon;
 
-		case 'fullscreen':
-			switch (CView::getLayoutMode()) {
-				case ZBX_LAYOUT_KIOSKMODE:
-					$icon = (new CButton(null, '&nbsp;'))
-						->setTitle(_('Normal view'))
-						->setAttribute('data-layout-mode', ZBX_LAYOUT_NORMAL)
-						->addClass(ZBX_LAYOUT_MODE)
-						->addClass(ZBX_STYLE_BTN_DASHBRD_NORMAL)
-						->addClass(ZBX_STYLE_BTN_MIN);
-					break;
-
-				case ZBX_LAYOUT_FULLSCREEN:
-					$icon = (new CButton(null, '&nbsp;'))
-						->setTitle(_('Kiosk mode'))
-						->setAttribute('data-layout-mode', ZBX_LAYOUT_KIOSKMODE)
-						->addClass(ZBX_LAYOUT_MODE)
-						->addClass(ZBX_STYLE_BTN_KIOSK);
-					break;
-
-				default:
-					$icon = (new CButton(null, '&nbsp;'))
-						->setTitle(_('Fullscreen'))
-						->setAttribute('data-layout-mode', ZBX_LAYOUT_FULLSCREEN)
-						->addClass(ZBX_LAYOUT_MODE)
-						->addClass(ZBX_STYLE_BTN_MAX);
+		case 'kioskmode':
+			if ($params['mode'] == ZBX_LAYOUT_KIOSKMODE) {
+				$icon = (new CButton(null, '&nbsp;'))
+					->setTitle(_('Normal view'))
+					->setAttribute('data-layout-mode', ZBX_LAYOUT_NORMAL)
+					->addClass(ZBX_LAYOUT_MODE)
+					->addClass(ZBX_STYLE_BTN_DASHBRD_NORMAL)
+					->addClass(ZBX_STYLE_BTN_MIN);
+			}
+			else {
+				$icon = (new CButton(null, '&nbsp;'))
+					->setTitle(_('Kiosk mode'))
+					->setAttribute('data-layout-mode', ZBX_LAYOUT_KIOSKMODE)
+					->addClass(ZBX_LAYOUT_MODE)
+					->addClass(ZBX_STYLE_BTN_KIOSK);
 			}
 
 			return $icon;
@@ -294,9 +284,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 		}
 
 		$breadcrumbs->addItem([
-			new CSpan(
-				new CLink(_('All templates'), 'templates.php?templateid='.$db_host['templateid'].url_param('groupid'))
-			),
+			new CSpan(new CLink(_('All templates'), new CUrl('templates.php'))),
 			'/',
 			$template
 		]);
@@ -342,7 +330,7 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 		}
 
 		$breadcrumbs->addItem([
-			new CSpan(new CLink(_('All hosts'), 'hosts.php?hostid='.$db_host['hostid'].url_param('groupid'))),
+			new CSpan(new CLink(_('All hosts'), new CUrl('hosts.php'))),
 			'/',
 			$host
 		]);
@@ -359,13 +347,18 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 	$content_menu = (new CList())
 		->setAttribute('role', 'navigation')
 		->setAttribute('aria-label', _('Content menu'));
+
 	/*
 	 * the count of rows
 	 */
 	if ($lld_ruleid == 0) {
 		// applications
 		$applications = new CSpan([
-			new CLink(_('Applications'), 'applications.php?hostid='.$db_host['hostid']),
+			new CLink(_('Applications'),
+				(new CUrl('applications.php'))
+					->setArgument('filter_set', '1')
+					->setArgument('filter_hostids', [$db_host['hostid']])
+			),
 			CViewHelper::showNum($db_host['applications'])
 		]);
 		if ($current_element == 'applications') {
@@ -403,7 +396,10 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 
 		// graphs
 		$graphs = new CSpan([
-			new CLink(_('Graphs'), 'graphs.php?hostid='.$db_host['hostid']),
+			new CLink(_('Graphs'), (new CUrl('graphs.php'))
+				->setArgument('filter_set', '1')
+				->setArgument('filter_hostids', [$db_host['hostid']])
+			),
 			CViewHelper::showNum($db_host['graphs'])
 		]);
 		if ($current_element == 'graphs') {
@@ -425,7 +421,10 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 
 		// discovery rules
 		$lld_rules = new CSpan([
-			new CLink(_('Discovery rules'), 'host_discovery.php?hostid='.$db_host['hostid']),
+			new CLink(_('Discovery rules'), (new CUrl('host_discovery.php'))
+				->setArgument('filter_set', '1')
+				->setArgument('filter_hostids', [$db_host['hostid']])
+			),
 			CViewHelper::showNum($db_host['discoveries'])
 		]);
 		if ($current_element == 'discoveries') {
@@ -435,7 +434,11 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 
 		// web scenarios
 		$http_tests = new CSpan([
-			new CLink(_('Web scenarios'), 'httpconf.php?hostid='.$db_host['hostid']),
+			new CLink(_('Web scenarios'),
+				(new CUrl('httpconf.php'))
+					->setArgument('filter_set', '1')
+					->setArgument('filter_hostids', [$db_host['hostid']])
+			),
 			CViewHelper::showNum($db_host['httpTests'])
 		]);
 		if ($current_element == 'web') {
@@ -457,7 +460,10 @@ function get_header_host_table($current_element, $hostid, $lld_ruleid = 0) {
 
 		$list->addItem([
 			(new CSpan())->addItem(
-				new CLink(_('Discovery list'), 'host_discovery.php?hostid='.$db_host['hostid'].url_param('groupid'))
+				new CLink(_('Discovery list'), (new CUrl('host_discovery.php'))
+					->setArgument('filter_set', '1')
+					->setArgument('filter_hostids', [$db_host['hostid']])
+				)
 			),
 			'/',
 			$discovery_rule
@@ -740,6 +746,37 @@ function getItemLifetimeIndicator($current_time, $ts_delete) {
 	return makeWarningIcon($warning);
 }
 
+function makeServerStatusOutput() {
+	return (new CTag('output', true))
+		->setId('msg-global-footer')
+		->addClass(ZBX_STYLE_MSG_GLOBAL_FOOTER)
+		->addClass(ZBX_STYLE_MSG_WARNING);
+}
+
+/**
+* Make logo of the specified type.
+*
+* @param int $type  LOGO_TYPE_NORMAL | LOGO_TYPE_SIDEBAR | LOGO_TYPE_SIDEBAR_COMPACT.
+*
+* @return CTag
+*/
+function makeLogo(int $type): ?CTag {
+	static $zabbix_logo_classes = [
+		LOGO_TYPE_NORMAL => ZBX_STYLE_ZABBIX_LOGO,
+		LOGO_TYPE_SIDEBAR => ZBX_STYLE_ZABBIX_SIDEBAR_LOGO,
+		LOGO_TYPE_SIDEBAR_COMPACT => ZBX_STYLE_ZABBIX_SIDEBAR_LOGO_COMPACT
+	];
+
+	$brand_logo = CBrandHelper::getLogo($type);
+
+	if ($brand_logo !== null) {
+		return (new CImg($brand_logo));
+	}
+	else {
+		return (new CDiv())->addClass($zabbix_logo_classes[$type]);
+	}
+}
+
 /**
  * Renders a page footer.
  *
@@ -748,35 +785,87 @@ function getItemLifetimeIndicator($current_time, $ts_delete) {
  * @return CDiv
  */
 function makePageFooter($with_version = true) {
-	return (new CTag('footer', true, CBrandHelper::getFooterContent($with_version)))->setAttribute('role', 'contentinfo');
+	return (new CTag('footer', true, CBrandHelper::getFooterContent($with_version)))
+		->setAttribute('role', 'contentinfo');
 }
 
 /**
- * Renders a drop-down menu for the Administration->General section.
+ * Get drop-down submenu item list for the Administration->General section.
  *
- * @param string $selected
- *
- * @return CComboBox
+ * @return array  Menu definition for CWidget::setTitleSubmenu.
  */
-function makeAdministrationGeneralMenu($selected) {
+function getAdministrationGeneralSubmenu() {
+	$gui_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'gui.edit')
+		->getUrl();
+
 	$autoreg_url = (new CUrl('zabbix.php'))
 		->setArgument('action', 'autoreg.edit')
 		->getUrl();
 
-	return new CComboBox('configDropDown', $selected, 'redirect(this.options[this.selectedIndex].value);', [
-		'adm.gui.php' => _('GUI'),
-		$autoreg_url => _('Auto registration'),
-		'adm.housekeeper.php' => _('Housekeeping'),
-		'adm.images.php' => _('Images'),
-		'adm.iconmapping.php' => _('Icon mapping'),
-		'adm.regexps.php' => _('Regular expressions'),
-		'adm.macros.php' => _('Macros'),
-		'adm.valuemapping.php' => _('Value mapping'),
-		'adm.workingtime.php' => _('Working time'),
-		'adm.triggerseverities.php' => _('Trigger severities'),
-		'adm.triggerdisplayoptions.php' => _('Trigger displaying options'),
-		'adm.other.php' => _('Other')
-	]);
+	$housekeeping_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'housekeeping.edit')
+		->getUrl();
+
+	$image_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'image.list')
+		->getUrl();
+
+	$iconmap_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'iconmap.list')
+		->getUrl();
+
+	$regex_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'regex.list')
+		->getUrl();
+
+	$macros_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'macros.edit')
+		->getUrl();
+
+	$valuemap_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'valuemap.list')
+		->getUrl();
+
+	$workingtime_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'workingtime.edit')
+		->getUrl();
+
+	$trigseverity_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'trigseverity.edit')
+		->getUrl();
+
+	$trigdisplay_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'trigdisplay.edit')
+		->getUrl();
+
+	$modules_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'module.list')
+		->getUrl();
+
+	$miscconfig_url = (new CUrl('zabbix.php'))
+		->setArgument('action', 'miscconfig.edit')
+		->getUrl();
+
+	return [
+		'main_section' => [
+			'items' => [
+				$gui_url          => _('GUI'),
+				$autoreg_url      => _('Autoregistration'),
+				$housekeeping_url => _('Housekeeping'),
+				$image_url        => _('Images'),
+				$iconmap_url      => _('Icon mapping'),
+				$regex_url        => _('Regular expressions'),
+				$macros_url       => _('Macros'),
+				$valuemap_url     => _('Value mapping'),
+				$workingtime_url  => _('Working time'),
+				$trigseverity_url => _('Trigger severities'),
+				$trigdisplay_url  => _('Trigger displaying options'),
+				$modules_url      => _('Modules'),
+				$miscconfig_url   => _('Other')
+			]
+		]
+	];
 }
 
 /**
@@ -950,24 +1039,6 @@ function makeWarningIcon($error) {
 		->setHint($error);
 }
 
-/**
- * Renders a debug button
- *
- * @return CButton
- */
-function makeDebugButton() {
-	return (new CDiv(
-		(new CLink(_('Debug'), '#debug'))
-			->onClick("javascript: if (!isset('state', this)) { this.state = 'none'; }".
-				"this.state = (this.state == 'none' ? 'block' : 'none');".
-				"jQuery(this)".
-					".text(this.state == 'none' ? ".CJs::encodeJson(_('Debug'))." : ".CJs::encodeJson(_('Hide debug')).")".
-					".blur();".
-				"showHideByName('zbx_debug_info', this.state);"
-			)
-	))->addClass(ZBX_STYLE_BTN_DEBUG);
-}
-
 /**
  * Returns css for trigger severity backgrounds.
  *
@@ -995,7 +1066,7 @@ function getTriggerSeverityCss($config) {
 
 	foreach ($severities as $class => $color) {
 		$css .= '.'.$class.', .'.$class.' input[type="radio"]:checked + label, .'.$class.':before, .flh-'.$class.
-			', .status-'.$class.' { background-color: #'.$color.' }'."\n";
+			', .status-'.$class.', .status-'.$class.':before { background-color: #'.$color.' }'."\n";
 	}
 
 	return $css;
